import os
import rb, rhythmdb
import gtk, gobject
import pango
import webkit
import re
import urllib2
from mako.template import Template


albumart_search_ui = """
<ui>
    <toolbar name="ToolBar">
        <toolitem name="AlbumArtSearch" action="ToggleAlbumArtSearch" />
    </toolbar>
</ui>
"""

class AlbumArtSearch (gobject.GObject) :

    def __init__ (self, shell, plugin) :
        gobject.GObject.__init__ (self)
        self.shell = shell
        self.sp = shell.get_player ()
        self.db = shell.get_property ('db')
        self.plugin = plugin

        self.file = ""
        self.basepath = 'file://' + os.path.split(plugin.find_file('AlbumArtSearch.py'))[0]
        self.load_templates()

        self.current_artist = None
        self.current_album = None
        self.current_song = None
        self.visible = True

        self.init_gui()
        self.connect_signals()


        self.action = ('ToggleAlbumArtSearch',gtk.STOCK_DND_MULTIPLE, _('Toggle album art search pane'),
                None, _('Change the visibility of album art search pane'),
                self.toggle_visibility, True)
        self.action_group = gtk.ActionGroup('AlbumArtSearchActions')
        self.action_group.add_toggle_actions([self.action])
        uim = self.shell.get_ui_manager()
        uim.insert_action_group (self.action_group, 0)
        self.ui_id = uim.add_ui_from_string(albumart_search_ui)
        uim.ensure_update()

    def connect_signals(self) :
        self.player_cb_ids = ( self.sp.connect ('playing-changed', self.playing_changed_cb), self.sp.connect ('playing-song-changed', self.playing_changed_cb))
        self.albumartbutton.connect('clicked', self.set_album_art)

    def playing_changed_cb (self, playing, user_data) :
        playing_entry = None
        if self.sp:
            playing_entry = self.sp.get_playing_entry ()
        if playing_entry is None :
            return

        playing_artist = self.db.entry_get (playing_entry, rhythmdb.PROP_ARTIST)
        playing_album = self.db.entry_get (playing_entry, rhythmdb.PROP_ALBUM)
        playing_title = self.db.entry_get (playing_entry, rhythmdb.PROP_TITLE)

        if playing_album.upper() == "UNKNOWN":
            self.current_album = ""
        if playing_artist.upper() == "UNKNOWN":
            self.current_artist = ""
        if not(self.current_album == "" and self.current_artist == ""):
            self.current_artist = playing_artist.replace ('&', '&amp;')
            self.current_album = playing_album.replace ('&', '&amp;')
            self.current_title = playing_title.replace ('&', '&amp;')
            self.render_album_art_search(self.current_artist, self.current_album, self.current_title)

    def set_album_art(self, button) :
        tburl = self.webview.get_main_frame().get_title()
        if not tburl:
            return
        print "Url: " + tburl
        filep = tburl.split('/')
        filepath = filep[len(filep)-1]
        (filen, filextn) = os.path.splitext(filepath)
        if filextn not in (".jpg", ".jpeg"):
            print "Only jpeg images are allowed as album cover"
            return
        request = urllib2.Request(tburl)
        opener = urllib2.build_opener()
        image = None
        try:
            image = opener.open(request).read()
        except:
            print "Failed to download image"
        filename = os.environ['HOME']+"/.cache/rhythmbox/covers/" + self.current_artist + " - " + self.current_album + ".jpg"
        output = open(filename, 'w')
        output.write(image)
        output.close()

    def render_album_art_search(self, artistname, albumname, currentitle):
        self.file = self.template.render (artist = artistname, album = albumname, title = currentitle, stylesheet = self.styles )
        self.webview.load_string (self.file, 'text/html', 'utf-8', self.basepath)

    def deactivate (self, shell) :
        self.shell = None
        self.sp = None
        self.db = None
        self.plugin = None
        shell.remove_widget (self.vbox, rb.SHELL_UI_LOCATION_RIGHT_SIDEBAR)
        uim = shell.get_ui_manager ()
        uim.remove_ui (self.ui_id)
        uim.remove_action_group (self.action_group)

    def toggle_visibility (self, action) :
        if not self.visible :
            self.shell.add_widget (self.vbox, rb.SHELL_UI_LOCATION_RIGHT_SIDEBAR, expand=True)
            self.visible = True
        else :
            self.shell.remove_widget (self.vbox, rb.SHELL_UI_LOCATION_RIGHT_SIDEBAR)
            self.visible = False

    def load_templates(self):
        self.path = self.plugin.find_file('tmpl/albumartsearch-tmpl.html')
        self.template = Template (filename = self.path, module_directory = '/tmp/')
        self.styles = self.basepath + '/tmpl/main.css'

    def init_gui(self) :
        self.vbox = gtk.VBox()

        #---- set up webkit pane -----#
        self.webview = webkit.WebView()
        self.scroll = gtk.ScrolledWindow()
        self.scroll.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        self.scroll.set_shadow_type(gtk.SHADOW_IN)
        self.scroll.add( self.webview )
        self.albumartbutton = gtk.Button (_("Set as Album Art"))

        self.vbox.pack_start(self.scroll, expand = True)
        self.vbox.pack_start(self.albumartbutton, expand = False)

        #---- pack everything into side panel ----#
        self.vbox.show_all()
        self.vbox.set_size_request(200, -1)
        self.shell.add_widget (self.vbox, rb.SHELL_UI_LOCATION_RIGHT_SIDEBAR, expand=True)

